# frozen_string_literal: true

require 'json'
require 'ostruct'
require 'erb'
require 'open3'
require 'zfs'

module Jailofficer
  # defines a singe jail
  class Jail
    attr_accessor :options
    attr_accessor :name
    attr_accessor :cfgfile

    def initialize(name)
      @name = name
      @options = { reserved: [] }
      fileselector
      loadfile if File.file?(cfgfile)
    end

    def set(key, value)
      @options[key] = value
      writefile
    end

    def get(key)
      @options[key]
    end

    def restart
      start if stop
    end

    def start
      execute("service jail start #{@name}")
    end

    def stop
      execute("service jail stop #{@name}")
    end

    def running?
      running = []
      getjails['running'].each do |jail|
        running << jail['hostname']
      end
      running.include?(@name)
    end

    def create
      writefile
      rc_add
      zfs_create
    end

    def destroy
      rc_del
      zfs_destroy
    end

    def enable
      rc_add
    end

    def disable
      rc_del
    end

    private

    def rc_add
      Sysrc.set('jail_list', '+=', @name)
    end

    def rc_del
      Sysrc.set('jail_list', '-=', @name)
    end

    def zfs_create
      fs = ZFS(format('%<base>s/%<name>s', { base: Config::BASEVOLUME, name: @name }))
      fs.create until fs.exist?
    end

    def zfs_destroy
      fs = ZFS(format('%<base>s/%<name>s', { base: Config::BASEVOLUME, name: @name }))
      fs.destroy! if fs.exist?
    end

    def execute(command)
      shell_env = 'LANG=C'
      cmd = "#{shell_env} #{command}"
      _stdout, _stderr, status = Open3.capture3(cmd)
      status.exitstatus.zero?
    end

    def writefile
      cfgcontent = ERB.new(template).result(binding).remove_empty_lines
      puts cfgcontent
      File.open(@cfgfile, 'w') { |file| file.write(cfgcontent) }
    end

    def fileselector
      @cfgfile = "#{Config::JCONFDIR}/jail.#{@name}.conf"
    end

    def loadfile
      filelines = File.readlines(@cfgfile)
      filelines.each do |line|
        if line.match?('=')
          sl = line.split('=')
          @options[sl[0].strip] = sl[1].strip.chop
        else
          @options[:reserved].append(line.strip.chop)
        end
      end
      @options[:reserved] = @options[:reserved].drop(1).clip
      @name = filelines[0].strip.partition(' ').first
    end

    def template
      %{
        <%= @name %> {
          <% @options[:reserved].each do |option| %>
            <%= option %>;
          <% end %>
          <% @options.except(:reserved).each do |key, value| %>
            <%= key %> = <%= value %>;
          <% end %>
        }
      }
    end

    def getjails
      list_raw = `sysrc jail_list`.split
      running_raw = JSON.parse(`jls --libxo json`)
      {
        list_raw[0].chop => list_raw.drop(1),
        'running' => running_raw['jail-information']['jail']
      }
    end
  end
end
